/*
 * Copyright (c) 1999, 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * ToOutChan.{cc,hh} -- Send packets to a OSKit netio.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "tooutchan.hh"
#include "error.hh"
#include "packet.hh"
#include "confparse.hh"
#include "glue.hh"
#include <unistd.h>

extern "C" {
#include <mempool.h>
}

ToOutChan::ToOutChan()
  : Element(1, 0)
{
}

ToOutChan::~ToOutChan()
{
}

ToOutChan *
ToOutChan::clone() const
{
	return new ToOutChan;
}

int
ToOutChan::configure(const String &, ErrorHandler *)
{
	return 0;
}

int
ToOutChan::initialize(ErrorHandler *)
{
	return 0;
}

/*
 * Call back to free the packet memory when we create an external version
 * Does nothing cause the memory is freed up in the Click packet destructor.
 * This will need to change when nodeos packets are ref counted.
 */
extern "C" void
ToOutChan_packet_free(channel_t *, void *, size_t)
{
}

void
ToOutChan::push(int, Packet *p)
{
	/*
	 * Kludgey. 
	 */
	packet_t	*nodeos_packet = p->nodeos_packet();

	if (! nodeos_packet) {
		int	err;
		
		err = packet_create_external(flow_current(), _chan,
				p->data(), p->length(), 0,
				ToOutChan_packet_free,
				p->headroom() + p->length() + p->tailroom(),
				&nodeos_packet);

		if (err) {
			click_chatter("ToOutChan::push: "
				      "Packet creation failed!\n");
			return;
		}

		/*
		 * The packet have a ref count > than 1, so someone else
		 * is holding it. Give it the nodeos packet pointer so
		 * we don't have to go through this again.
		 */
		p->set_nodeos_packet(nodeos_packet);
	}
	channel_send(_chan, nodeos_packet);
	p->kill();
}

bool
ToOutChan::wants_packet_upstream() const
{
	return input_is_pull(0);
}

void
ToOutChan::run_scheduled()
{
	while (Packet *p = input(0).pull())
		push(0, p);
}

EXPORT_ELEMENT(ToOutChan)
